<?php

namespace Drupal\lms\Entity\Handlers;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;

/**
 * Access controller for the User lesson status entity.
 *
 * @see \Drupal\lms\Entity\LessonStatus.
 */
class LessonStatusAccessControlHandler extends EntityAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    // Check if user can view own results.
    // @todo Define permission, implement per-lesson results.
    if ($entity->getOwnerId() == $account->id() && $account->hasPermission('view own lesson results')
          && $operation == 'view' && $entity->isFinished()) {
      return AccessResult::allowed();
    }

    // Unknown operation, no opinion.
    return AccessResult::neutral();
  }

  /**
   * {@inheritdoc}
   */
  protected function checkCreateAccess(AccountInterface $account, array $context, $entity_bundle = NULL) {
    // Creation from API only.
    return AccessResult::forbidden();
  }

}
