<?php

declare(strict_types=1);

namespace Drupal\lms\Exception;

use Drupal\lms\Entity\CourseStatus;
use Drupal\lms\Plugin\Field\FieldType\LMSReferenceItem;

/**
 * Defines a training exceprion class.
 *
 * Error codes:
 *     1 - previous lesson with minimum score set or mandatory flag
 *         not started,
 *     2 - previous lesson with mandatory flag not finished,
 *     3 - previous lesson minimum score not achieved.
 */
final class TrainingException extends \Exception {
  public const REQUIRED_NOT_STARTED = 1;
  public const REQUIRED_NOT_FINISHED = 2;
  public const INSUFFICIENT_SCORE = 3;
  public const NO_LESSONS = 4;
  public const LESSON_REMOVED = 5;

  public function __construct(
    private readonly ?CourseStatus $lpStatus,
    private readonly ?LMSReferenceItem $step,
    int $code,
  ) {
    $messages = [
      self::REQUIRED_NOT_STARTED => 'Required lesson not started',
      self::REQUIRED_NOT_FINISHED => 'Required lesson not finished',
      self::INSUFFICIENT_SCORE => 'Required score on previous lesson not met',
      self::NO_LESSONS => 'This learning path has no lessons',
      self::LESSON_REMOVED => 'The requested lesson is not a part of the course',
    ];

    if (!\array_key_exists($code, $messages)) {
      throw new \InvalidArgumentException('Invalid error code provided.');
    }

    parent::__construct($messages[$code], $code);
  }

  /**
   * Lesson getter.
   */
  public function getStep(): ?LMSReferenceItem {
    return $this->step;
  }

  /**
   * LP status getter.
   */
  public function getLpStatus(): ?CourseStatus {
    return $this->lpStatus;
  }

}
